// Generated by Ignite ignite.com/cli

import { StdFee } from "@cosmjs/launchpad";
import { SigningStargateClient, DeliverTxResponse } from "@cosmjs/stargate";
import { EncodeObject, GeneratedType, OfflineSigner, Registry } from "@cosmjs/proto-signing";
import { msgTypes } from './registry';
import { IgniteClient } from "../client"
import { MissingWalletError } from "../helpers"
import { Api } from "./rest";
import { MsgCreateIssuer } from "./types/issuer/tx";
import { MsgBurnToken } from "./types/issuer/tx";
import { MsgIssueUserCredential } from "./types/issuer/tx";
import { MsgMintToken } from "./types/issuer/tx";
import { MsgPauseToken } from "./types/issuer/tx";


export { MsgCreateIssuer, MsgBurnToken, MsgIssueUserCredential, MsgMintToken, MsgPauseToken };

type sendMsgCreateIssuerParams = {
  value: MsgCreateIssuer,
  fee?: StdFee,
  memo?: string
};

type sendMsgBurnTokenParams = {
  value: MsgBurnToken,
  fee?: StdFee,
  memo?: string
};

type sendMsgIssueUserCredentialParams = {
  value: MsgIssueUserCredential,
  fee?: StdFee,
  memo?: string
};

type sendMsgMintTokenParams = {
  value: MsgMintToken,
  fee?: StdFee,
  memo?: string
};

type sendMsgPauseTokenParams = {
  value: MsgPauseToken,
  fee?: StdFee,
  memo?: string
};


type msgCreateIssuerParams = {
  value: MsgCreateIssuer,
};

type msgBurnTokenParams = {
  value: MsgBurnToken,
};

type msgIssueUserCredentialParams = {
  value: MsgIssueUserCredential,
};

type msgMintTokenParams = {
  value: MsgMintToken,
};

type msgPauseTokenParams = {
  value: MsgPauseToken,
};


export const registry = new Registry(msgTypes);

const defaultFee = {
  amount: [],
  gas: "200000",
};

interface TxClientOptions {
  addr: string
	prefix: string
	signer?: OfflineSigner
}

export const txClient = ({ signer, prefix, addr }: TxClientOptions = { addr: "http://localhost:26657", prefix: "cosmos" }) => {

  return {
		
		async sendMsgCreateIssuer({ value, fee, memo }: sendMsgCreateIssuerParams): Promise<DeliverTxResponse> {
			if (!signer) {
					throw new Error('TxClient:sendMsgCreateIssuer: Unable to sign Tx. Signer is not present.')
			}
			try {			
				const { address } = (await signer.getAccounts())[0]; 
				const signingClient = await SigningStargateClient.connectWithSigner(addr,signer,{registry, prefix});
				let msg = this.msgCreateIssuer({ value: MsgCreateIssuer.fromPartial(value) })
				return await signingClient.signAndBroadcast(address, [msg], fee ? fee : defaultFee, memo)
			} catch (e: any) {
				throw new Error('TxClient:sendMsgCreateIssuer: Could not broadcast Tx: '+ e.message)
			}
		},
		
		async sendMsgBurnToken({ value, fee, memo }: sendMsgBurnTokenParams): Promise<DeliverTxResponse> {
			if (!signer) {
					throw new Error('TxClient:sendMsgBurnToken: Unable to sign Tx. Signer is not present.')
			}
			try {			
				const { address } = (await signer.getAccounts())[0]; 
				const signingClient = await SigningStargateClient.connectWithSigner(addr,signer,{registry, prefix});
				let msg = this.msgBurnToken({ value: MsgBurnToken.fromPartial(value) })
				return await signingClient.signAndBroadcast(address, [msg], fee ? fee : defaultFee, memo)
			} catch (e: any) {
				throw new Error('TxClient:sendMsgBurnToken: Could not broadcast Tx: '+ e.message)
			}
		},
		
		async sendMsgIssueUserCredential({ value, fee, memo }: sendMsgIssueUserCredentialParams): Promise<DeliverTxResponse> {
			if (!signer) {
					throw new Error('TxClient:sendMsgIssueUserCredential: Unable to sign Tx. Signer is not present.')
			}
			try {			
				const { address } = (await signer.getAccounts())[0]; 
				const signingClient = await SigningStargateClient.connectWithSigner(addr,signer,{registry, prefix});
				let msg = this.msgIssueUserCredential({ value: MsgIssueUserCredential.fromPartial(value) })
				return await signingClient.signAndBroadcast(address, [msg], fee ? fee : defaultFee, memo)
			} catch (e: any) {
				throw new Error('TxClient:sendMsgIssueUserCredential: Could not broadcast Tx: '+ e.message)
			}
		},
		
		async sendMsgMintToken({ value, fee, memo }: sendMsgMintTokenParams): Promise<DeliverTxResponse> {
			if (!signer) {
					throw new Error('TxClient:sendMsgMintToken: Unable to sign Tx. Signer is not present.')
			}
			try {			
				const { address } = (await signer.getAccounts())[0]; 
				const signingClient = await SigningStargateClient.connectWithSigner(addr,signer,{registry, prefix});
				let msg = this.msgMintToken({ value: MsgMintToken.fromPartial(value) })
				return await signingClient.signAndBroadcast(address, [msg], fee ? fee : defaultFee, memo)
			} catch (e: any) {
				throw new Error('TxClient:sendMsgMintToken: Could not broadcast Tx: '+ e.message)
			}
		},
		
		async sendMsgPauseToken({ value, fee, memo }: sendMsgPauseTokenParams): Promise<DeliverTxResponse> {
			if (!signer) {
					throw new Error('TxClient:sendMsgPauseToken: Unable to sign Tx. Signer is not present.')
			}
			try {			
				const { address } = (await signer.getAccounts())[0]; 
				const signingClient = await SigningStargateClient.connectWithSigner(addr,signer,{registry, prefix});
				let msg = this.msgPauseToken({ value: MsgPauseToken.fromPartial(value) })
				return await signingClient.signAndBroadcast(address, [msg], fee ? fee : defaultFee, memo)
			} catch (e: any) {
				throw new Error('TxClient:sendMsgPauseToken: Could not broadcast Tx: '+ e.message)
			}
		},
		
		
		msgCreateIssuer({ value }: msgCreateIssuerParams): EncodeObject {
			try {
				return { typeUrl: "/allinbits.cosmoscash.issuer.MsgCreateIssuer", value: MsgCreateIssuer.fromPartial( value ) }  
			} catch (e: any) {
				throw new Error('TxClient:MsgCreateIssuer: Could not create message: ' + e.message)
			}
		},
		
		msgBurnToken({ value }: msgBurnTokenParams): EncodeObject {
			try {
				return { typeUrl: "/allinbits.cosmoscash.issuer.MsgBurnToken", value: MsgBurnToken.fromPartial( value ) }  
			} catch (e: any) {
				throw new Error('TxClient:MsgBurnToken: Could not create message: ' + e.message)
			}
		},
		
		msgIssueUserCredential({ value }: msgIssueUserCredentialParams): EncodeObject {
			try {
				return { typeUrl: "/allinbits.cosmoscash.issuer.MsgIssueUserCredential", value: MsgIssueUserCredential.fromPartial( value ) }  
			} catch (e: any) {
				throw new Error('TxClient:MsgIssueUserCredential: Could not create message: ' + e.message)
			}
		},
		
		msgMintToken({ value }: msgMintTokenParams): EncodeObject {
			try {
				return { typeUrl: "/allinbits.cosmoscash.issuer.MsgMintToken", value: MsgMintToken.fromPartial( value ) }  
			} catch (e: any) {
				throw new Error('TxClient:MsgMintToken: Could not create message: ' + e.message)
			}
		},
		
		msgPauseToken({ value }: msgPauseTokenParams): EncodeObject {
			try {
				return { typeUrl: "/allinbits.cosmoscash.issuer.MsgPauseToken", value: MsgPauseToken.fromPartial( value ) }  
			} catch (e: any) {
				throw new Error('TxClient:MsgPauseToken: Could not create message: ' + e.message)
			}
		},
		
	}
};

interface QueryClientOptions {
  addr: string
}

export const queryClient = ({ addr: addr }: QueryClientOptions = { addr: "http://localhost:1317" }) => {
  return new Api({ baseUrl: addr });
};

class SDKModule {
	public query: ReturnType<typeof queryClient>;
	public tx: ReturnType<typeof txClient>;
	
	public registry: Array<[string, GeneratedType]>;

	constructor(client: IgniteClient) {		
	
		this.query = queryClient({ addr: client.env.apiURL });
		this.tx = txClient({ signer: client.signer, addr: client.env.rpcURL, prefix: client.env.prefix ?? "cosmos" });
	}
};

const Module = (test: IgniteClient) => {
	return {
		module: {
			AllinbitsCosmoscashIssuer: new SDKModule(test)
		},
		registry: msgTypes
  }
}
export default Module;