// Generated by Ignite ignite.com/cli

import { StdFee } from "@cosmjs/launchpad";
import { SigningStargateClient, DeliverTxResponse } from "@cosmjs/stargate";
import { EncodeObject, GeneratedType, OfflineSigner, Registry } from "@cosmjs/proto-signing";
import { msgTypes } from './registry';
import { IgniteClient } from "../client"
import { MissingWalletError } from "../helpers"
import { Api } from "./rest";
import { MsgIssueLicenseCredential } from "./types/regulator/tx";
import { MsgIssueRegistrationCredential } from "./types/regulator/tx";
import { MsgIssueRegulatorCredential } from "./types/regulator/tx";


export { MsgIssueLicenseCredential, MsgIssueRegistrationCredential, MsgIssueRegulatorCredential };

type sendMsgIssueLicenseCredentialParams = {
  value: MsgIssueLicenseCredential,
  fee?: StdFee,
  memo?: string
};

type sendMsgIssueRegistrationCredentialParams = {
  value: MsgIssueRegistrationCredential,
  fee?: StdFee,
  memo?: string
};

type sendMsgIssueRegulatorCredentialParams = {
  value: MsgIssueRegulatorCredential,
  fee?: StdFee,
  memo?: string
};


type msgIssueLicenseCredentialParams = {
  value: MsgIssueLicenseCredential,
};

type msgIssueRegistrationCredentialParams = {
  value: MsgIssueRegistrationCredential,
};

type msgIssueRegulatorCredentialParams = {
  value: MsgIssueRegulatorCredential,
};


export const registry = new Registry(msgTypes);

const defaultFee = {
  amount: [],
  gas: "200000",
};

interface TxClientOptions {
  addr: string
	prefix: string
	signer?: OfflineSigner
}

export const txClient = ({ signer, prefix, addr }: TxClientOptions = { addr: "http://localhost:26657", prefix: "cosmos" }) => {

  return {
		
		async sendMsgIssueLicenseCredential({ value, fee, memo }: sendMsgIssueLicenseCredentialParams): Promise<DeliverTxResponse> {
			if (!signer) {
					throw new Error('TxClient:sendMsgIssueLicenseCredential: Unable to sign Tx. Signer is not present.')
			}
			try {			
				const { address } = (await signer.getAccounts())[0]; 
				const signingClient = await SigningStargateClient.connectWithSigner(addr,signer,{registry, prefix});
				let msg = this.msgIssueLicenseCredential({ value: MsgIssueLicenseCredential.fromPartial(value) })
				return await signingClient.signAndBroadcast(address, [msg], fee ? fee : defaultFee, memo)
			} catch (e: any) {
				throw new Error('TxClient:sendMsgIssueLicenseCredential: Could not broadcast Tx: '+ e.message)
			}
		},
		
		async sendMsgIssueRegistrationCredential({ value, fee, memo }: sendMsgIssueRegistrationCredentialParams): Promise<DeliverTxResponse> {
			if (!signer) {
					throw new Error('TxClient:sendMsgIssueRegistrationCredential: Unable to sign Tx. Signer is not present.')
			}
			try {			
				const { address } = (await signer.getAccounts())[0]; 
				const signingClient = await SigningStargateClient.connectWithSigner(addr,signer,{registry, prefix});
				let msg = this.msgIssueRegistrationCredential({ value: MsgIssueRegistrationCredential.fromPartial(value) })
				return await signingClient.signAndBroadcast(address, [msg], fee ? fee : defaultFee, memo)
			} catch (e: any) {
				throw new Error('TxClient:sendMsgIssueRegistrationCredential: Could not broadcast Tx: '+ e.message)
			}
		},
		
		async sendMsgIssueRegulatorCredential({ value, fee, memo }: sendMsgIssueRegulatorCredentialParams): Promise<DeliverTxResponse> {
			if (!signer) {
					throw new Error('TxClient:sendMsgIssueRegulatorCredential: Unable to sign Tx. Signer is not present.')
			}
			try {			
				const { address } = (await signer.getAccounts())[0]; 
				const signingClient = await SigningStargateClient.connectWithSigner(addr,signer,{registry, prefix});
				let msg = this.msgIssueRegulatorCredential({ value: MsgIssueRegulatorCredential.fromPartial(value) })
				return await signingClient.signAndBroadcast(address, [msg], fee ? fee : defaultFee, memo)
			} catch (e: any) {
				throw new Error('TxClient:sendMsgIssueRegulatorCredential: Could not broadcast Tx: '+ e.message)
			}
		},
		
		
		msgIssueLicenseCredential({ value }: msgIssueLicenseCredentialParams): EncodeObject {
			try {
				return { typeUrl: "/allinbits.cosmoscash.regulator.MsgIssueLicenseCredential", value: MsgIssueLicenseCredential.fromPartial( value ) }  
			} catch (e: any) {
				throw new Error('TxClient:MsgIssueLicenseCredential: Could not create message: ' + e.message)
			}
		},
		
		msgIssueRegistrationCredential({ value }: msgIssueRegistrationCredentialParams): EncodeObject {
			try {
				return { typeUrl: "/allinbits.cosmoscash.regulator.MsgIssueRegistrationCredential", value: MsgIssueRegistrationCredential.fromPartial( value ) }  
			} catch (e: any) {
				throw new Error('TxClient:MsgIssueRegistrationCredential: Could not create message: ' + e.message)
			}
		},
		
		msgIssueRegulatorCredential({ value }: msgIssueRegulatorCredentialParams): EncodeObject {
			try {
				return { typeUrl: "/allinbits.cosmoscash.regulator.MsgIssueRegulatorCredential", value: MsgIssueRegulatorCredential.fromPartial( value ) }  
			} catch (e: any) {
				throw new Error('TxClient:MsgIssueRegulatorCredential: Could not create message: ' + e.message)
			}
		},
		
	}
};

interface QueryClientOptions {
  addr: string
}

export const queryClient = ({ addr: addr }: QueryClientOptions = { addr: "http://localhost:1317" }) => {
  return new Api({ baseUrl: addr });
};

class SDKModule {
	public query: ReturnType<typeof queryClient>;
	public tx: ReturnType<typeof txClient>;
	
	public registry: Array<[string, GeneratedType]>;

	constructor(client: IgniteClient) {		
	
		this.query = queryClient({ addr: client.env.apiURL });
		this.tx = txClient({ signer: client.signer, addr: client.env.rpcURL, prefix: client.env.prefix ?? "cosmos" });
	}
};

const Module = (test: IgniteClient) => {
	return {
		module: {
			AllinbitsCosmoscashRegulator: new SDKModule(test)
		},
		registry: msgTypes
  }
}
export default Module;