// Generated by Ignite ignite.com/cli

import { StdFee } from "@cosmjs/launchpad";
import { SigningStargateClient, DeliverTxResponse } from "@cosmjs/stargate";
import { EncodeObject, GeneratedType, OfflineSigner, Registry } from "@cosmjs/proto-signing";
import { msgTypes } from './registry';
import { IgniteClient } from "../client"
import { MissingWalletError } from "../helpers"
import { Api } from "./rest";
import { MsgRevokeCredential } from "./types/verifiable-credential/tx";
import { MsgIssueCredential } from "./types/verifiable-credential/tx";


export { MsgRevokeCredential, MsgIssueCredential };

type sendMsgRevokeCredentialParams = {
  value: MsgRevokeCredential,
  fee?: StdFee,
  memo?: string
};

type sendMsgIssueCredentialParams = {
  value: MsgIssueCredential,
  fee?: StdFee,
  memo?: string
};


type msgRevokeCredentialParams = {
  value: MsgRevokeCredential,
};

type msgIssueCredentialParams = {
  value: MsgIssueCredential,
};


export const registry = new Registry(msgTypes);

const defaultFee = {
  amount: [],
  gas: "200000",
};

interface TxClientOptions {
  addr: string
	prefix: string
	signer?: OfflineSigner
}

export const txClient = ({ signer, prefix, addr }: TxClientOptions = { addr: "http://localhost:26657", prefix: "cosmos" }) => {

  return {
		
		async sendMsgRevokeCredential({ value, fee, memo }: sendMsgRevokeCredentialParams): Promise<DeliverTxResponse> {
			if (!signer) {
					throw new Error('TxClient:sendMsgRevokeCredential: Unable to sign Tx. Signer is not present.')
			}
			try {			
				const { address } = (await signer.getAccounts())[0]; 
				const signingClient = await SigningStargateClient.connectWithSigner(addr,signer,{registry, prefix});
				let msg = this.msgRevokeCredential({ value: MsgRevokeCredential.fromPartial(value) })
				return await signingClient.signAndBroadcast(address, [msg], fee ? fee : defaultFee, memo)
			} catch (e: any) {
				throw new Error('TxClient:sendMsgRevokeCredential: Could not broadcast Tx: '+ e.message)
			}
		},
		
		async sendMsgIssueCredential({ value, fee, memo }: sendMsgIssueCredentialParams): Promise<DeliverTxResponse> {
			if (!signer) {
					throw new Error('TxClient:sendMsgIssueCredential: Unable to sign Tx. Signer is not present.')
			}
			try {			
				const { address } = (await signer.getAccounts())[0]; 
				const signingClient = await SigningStargateClient.connectWithSigner(addr,signer,{registry, prefix});
				let msg = this.msgIssueCredential({ value: MsgIssueCredential.fromPartial(value) })
				return await signingClient.signAndBroadcast(address, [msg], fee ? fee : defaultFee, memo)
			} catch (e: any) {
				throw new Error('TxClient:sendMsgIssueCredential: Could not broadcast Tx: '+ e.message)
			}
		},
		
		
		msgRevokeCredential({ value }: msgRevokeCredentialParams): EncodeObject {
			try {
				return { typeUrl: "/allinbits.cosmoscash.verifiablecredential.MsgRevokeCredential", value: MsgRevokeCredential.fromPartial( value ) }  
			} catch (e: any) {
				throw new Error('TxClient:MsgRevokeCredential: Could not create message: ' + e.message)
			}
		},
		
		msgIssueCredential({ value }: msgIssueCredentialParams): EncodeObject {
			try {
				return { typeUrl: "/allinbits.cosmoscash.verifiablecredential.MsgIssueCredential", value: MsgIssueCredential.fromPartial( value ) }  
			} catch (e: any) {
				throw new Error('TxClient:MsgIssueCredential: Could not create message: ' + e.message)
			}
		},
		
	}
};

interface QueryClientOptions {
  addr: string
}

export const queryClient = ({ addr: addr }: QueryClientOptions = { addr: "http://localhost:1317" }) => {
  return new Api({ baseUrl: addr });
};

class SDKModule {
	public query: ReturnType<typeof queryClient>;
	public tx: ReturnType<typeof txClient>;
	
	public registry: Array<[string, GeneratedType]>;

	constructor(client: IgniteClient) {		
	
		this.query = queryClient({ addr: client.env.apiURL });
		this.tx = txClient({ signer: client.signer, addr: client.env.rpcURL, prefix: client.env.prefix ?? "cosmos" });
	}
};

const Module = (test: IgniteClient) => {
	return {
		module: {
			AllinbitsCosmoscashVerifiablecredential: new SDKModule(test)
		},
		registry: msgTypes
  }
}
export default Module;